@@ -1,5 +1,7 @@
 leda_download<-function(url,skip_row,column,out_name){
-    base_url<-"http://www.leda-traitbase.org/LEDAportal/objects/Data_files/"
+#    base_url<-"http://www.leda-traitbase.org/LEDAportal/objects/Data_files/"
+    ##base_url<-"http://www.uni-oldenburg.de/en/landeco/research/projects/LEDA/Data Files/"
+    base_url<-"http://www.uni-oldenburg.de/fileadmin/user_upload/biologie/ag/landeco/download/LEDA/Data_files/"
     url<-paste(base_url,url,sep="")
     downloaded<-read.csv(url,row.names=NULL,skip=skip_row,sep=";",check.names="F")
     
